#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

import json

from onf_website.component.location import ILocationInfo, ILocationTarget
from pyams_content.component.paragraph.interfaces.map import have_gis
from pyams_gis.layer import GeoJSONLayer
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url


if have_gis:

    from zope.schema.fieldproperty import FieldProperty

    from pyams_content.component.paragraph.interfaces.map import IMapParagraph
    from pyams_content.features.renderer.interfaces import IContentRenderer
    from pyams_default_theme.component.paragraph.interfaces.map import \
        IMapParagraphDefaultRendererSettings
    from pyams_default_theme.features.renderer import BaseContentRenderer
    from pyams_gis.configuration import MapConfiguration
    from pyams_gis.interfaces.configuration import IMapConfiguration
    from pyams_gis.interfaces.utility import IMapManager
    from pyams_skin.layer import IPyAMSLayer
    from pyams_template.template import template_config
    from pyams_utils.adapter import adapter_config, get_annotation_adapter
    from pyams_utils.factory import factory_config
    from pyams_utils.registry import get_utility

    from pyams_default_theme import _


    #
    # Map paragraph default renderer settings
    #

    MAP_DEFAULT_RENDERER_SETTINGS_KEY = 'pyams_content.map.renderer:default'


    @factory_config(IMapParagraphDefaultRendererSettings)
    class MapParagraphDefaultRendererSettings(MapConfiguration):
        """Map paragraph default renderer settings"""

        _use_default_map_configuration = FieldProperty(IMapParagraphDefaultRendererSettings[
                                                           'use_default_map_configuration'])

        map_height = FieldProperty(IMapParagraphDefaultRendererSettings['map_height'])

        @property
        def use_default_map_configuration(self):
            return self._use_default_map_configuration

        @use_default_map_configuration.setter
        def use_default_map_configuration(self, value):
            self._use_default_map_configuration = value

        @property
        def no_use_default_map_configuration(self):
            return not bool(self.use_default_map_configuration)

        @no_use_default_map_configuration.setter
        def no_use_default_map_configuration(self, value):
            self.use_default_map_configuration = not bool(value)

        @property
        def configuration(self):
            if self.use_default_map_configuration:
                manager = get_utility(IMapManager)
                return IMapConfiguration(manager).get_configuration()
            return self.get_configuration()


    @adapter_config(context=IMapParagraph,
                    provides=IMapParagraphDefaultRendererSettings)
    def map_paragraph_default_renderer_settings_factory(context):
        """Map paragraph default renderer settings factory"""
        return get_annotation_adapter(context, MAP_DEFAULT_RENDERER_SETTINGS_KEY,
                                      IMapParagraphDefaultRendererSettings)


    #
    # Map paragraph default renderer
    #

    @adapter_config(name='default',
                    context=(IMapParagraph, IPyAMSLayer),
                    provides=IContentRenderer)
    @template_config(template='templates/map-default.pt', layer=IPyAMSLayer)
    class MapParagraphDefaultRenderer(BaseContentRenderer):
        """Map paragraph default renderer"""

        label = _("Default map renderer")

        settings_interface = IMapParagraphDefaultRendererSettings

        i18n_context_attrs = ('title', )

        def get_map_configuration(self):
            configuration = self.settings.configuration
            settings = self.settings
            if settings.display_context_forests:
                parent = get_parent(self.context, ILocationTarget)
                if parent is not None:
                    key = ICacheKeyValue(parent)
                    layer = GeoJSONLayer()
                    layer.name = '{}-forests'.format(key)
                    layer.url = absolute_url(parent, self.request, 'get-location-forests.json')
                    layer.style = json.dumps({
                        'color': '#00dd00',
                        'weight': '3',
                        'opacity': '0.5'
                    })
                    configuration.setdefault('layers', []).append(layer.get_configuration())
                    if settings.set_bounds_to_forests:
                        configuration['fitLayer'] = layer.name
            return json.dumps(configuration)

        def get_markers(self):
            if not self.context.display_marker:
                return None
            if self.context.use_context_location:
                parent = get_parent(self.context, ILocationTarget)
                if parent is None:
                    return None
                locations = ILocationInfo(parent).locations
                if not locations:
                    return None
            else:
                if not self.context.gps_location:
                    return None
                locations = [self.context.gps_location]
            if not locations:
                return None
            config = {
                'icon': {
                    'url': '/--static--/onf_website/images/leaflet-marker.png',
                    'size': [47, 59],
                    'anchor': [23, 58]
                },
                'clusterClass': 'onfCluster',
                'tooltipClass': 'onfTooltip',
                'markers': []
            }
            append = config['markers'].append
            for idx, location in enumerate(locations):
                append({
                    'id': '{}::{}'.format(ICacheKeyValue(self.context), idx),
                    'point': location.to_json()
                })
            return json.dumps(config)